/*	evt_queue.h	*/

#ifndef _EVT_QUEUE_H_
#define _EVT_QUEUE_H_

/*
 * Copyright 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 by
 *	University of New Hampshire Interoperability Laboratory
 *	121 Technology Drive, Suite 2
 *	Durham, NH 03824-4716
 *	phone: 1-603-862-0090
 *	fax: 1-603-862-4181
 *	e-mail: info@iol.unh.edu
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grant No. OCI-1127228.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

/*
 * The UNH IOL implementation of the Extended Sockets API (ES-API) Issue 1.0
 *
 * defined in:	The Interconnect Software Consortium Issue 1.0
 *		ISBN: 1-931624-52-6
 *		Published by The open Group, January 2005.
 *
 * all page references in this source code are to pages in that document.
 */

/*
 * Most of the coding in this file done by:
 *	Bob Russell
 *	rdr@iol.unh.edu
 */


#include "exs.h"
#include "list.h"


/* type of our internal pointers to queues */
typedef struct evt_queue *	our_qhandle_t;

/* set to non-zero to enable debug printing */
extern int _our_debug_flag;


/* Parameters for pending connects, but present in all connections */
struct pending_connect {
	our_qhandle_t qhandle;
	exs_ahandle_t		ahandle;
	int			err_numb;
	int			sockno;
	int			evt_type;
	int			old_state;
};


/* the number of bytes to allocate for strerror_r() buffer */
#define	_MAX_STRERROR_MALLOC	52

struct evt_queue
	{
	/* the list of full "struct exs_event" objects on this queue */
	struct our_list_head	full_list;
	unsigned int		waiting_for_notify;

	/* the list of empty "struct exs_event" objects on this queue */
	struct our_list_head	empty_list;

	/* if > 0, the enabled signal
	 * if <= 0, signalling is disabled
	 */
	int			signal_number;

	/* the number of "struct exs_event" objects allocated to this queue */
	unsigned int		nitems;

	/* the current depth value defined by user */
	unsigned int		depth;

	/* the number of pending events on this queue */
	volatile unsigned int	npending;

	/* added for polling */
	unsigned int		get_out_now;
	pthread_cond_t		get_out_notify;
	unsigned long		qnumb;
	};


extern our_qhandle_t
_evt_queue_init( unsigned int depth );


extern int
_evt_queue_deinit( our_qhandle_t qhandle );


/* disables signal generation for this queue
 * returns 0 if all ok, -1 and errno set on any error
 */
extern int
_evt_queue_disable_signal( our_qhandle_t qhandle );


/* enables signal generation for this queue whenever a new event is
 * enqueued on a previously empty queue
 * returns 0 if all ok, -1 and errno set on any error
 */
extern int
_evt_queue_enable_signal( our_qhandle_t qhandle, int signal_number );


/* checks to see if anyone is waiting on "full_notify"
 * if yes, broadcasts the condition variable
 * if not, doesn't
 *
 * always raises any non-zero signo
 * returns 0 if all ok, else -1 with errno set
 */
extern int
_evt_queue_do_notify( our_qhandle_t qhandle, int signo );


/* returns in vector up to count events posted to qhandle
 * returns the number actually copied into vector if all ok
 * returns -1 with errno set if any error
 *
 * FOR NOW, IGNORE TIMEOUT *** TO DO - fix this ***
 */
extern int
_evt_queue_dequeue_vector( our_qhandle_t qhandle, exs_event_t *vector,
			  int count, const struct timeval *timeout );


/* if qhandle's empty list is not itself empty,
 * then removes and returns first event struct on that list
 * else returns NULL
 */
extern exs_event_t *
_get_empty_evt_queue_event( our_qhandle_t qhandle, exs_qhandle_t qn );


extern int
_put_full_evt_queue_event( our_qhandle_t qhandle, exs_event_t *evt_ptr );


extern int
_swap_evt_to_full_evt_queue( our_qhandle_t qhandle, exs_event_t **evt_ptr );


/*
 * on entry, conn_pending->qhandle is known not to be NULL.
 * on exit, conn_pending->qhandle is always set to be NULL.
 *
 * if all ok, gets a new exs_evt_t item from qhandle's empty list,
 * fills it in with event info, and returns in qhandle and signo
 * the info necessary for caller to post this event
 *
 * returns 0 if all ok, else -1 with errno set (although result is never used!)
 */
extern int
_post_event_to_full_evt_queue( struct pending_connect *conn_pending,
				our_qhandle_t *qhandle, int *signo );


extern int
_get_evt_queue_depth( our_qhandle_t qhandle, int *result );


extern int
_get_evt_queue_signal( our_qhandle_t qhandle, struct exs_signal *sig);


extern int
_get_evt_queue_n_events( our_qhandle_t qhandle, int *result );

int
_evt_queue_add_pending(our_qhandle_t qhandle, unsigned int n);

int
_evt_queue_rm_pending(our_qhandle_t qhandle, int n);

extern our_qhandle_t
_cnv_qnumb_to_ptr( exs_qhandle_t qh );

extern exs_qhandle_t
_cnv_ptr_to_qnumb( our_qhandle_t qh );

#endif /* _EVT_QUEUE_H_ */

/* vi: set autoindent tabstop=8 shiftwidth=8 : */
